#!/usr/bin/perl -w 

# Database broker - provides access to MYSQL databases to Database components in Dana.
# Author: Roberto Rodrigues Filho
# August @ 2015

use DBI; 
use IO::Socket::INET;

# auto-flush on socket
$| = 1;
 
# creating a listening socket
my $socket = new IO::Socket::INET (
    LocalHost => '0.0.0.0',
    LocalPort => '7777',
    Proto => 'tcp',
    Listen => 5,
    Reuse => 1
);

# reading from configuration file
# database information
open (MYFILE, 'database.config'); 
while ($fileContent = <MYFILE>) {
    $fileContent =~ s/^\s+|\s+$//g;
    @file_lines = split(/:/, $fileContent);
    push(@configInfo, $file_lines[1]);
} 
close (MYFILE);

# trimming the white spaces
$configInfo[0] =~ s/^\s+|\s+$//g;
$configInfo[1] =~ s/^\s+|\s+$//g;
$configInfo[2] =~ s/^\s+|\s+$//g;

# creating database
$dbh = DBI->connect('dbi:mysql:'.$configInfo[0],$configInfo[1],$configInfo[2]) or die "Connection Error: $DBI::errstr\n"; 

die "cannot create socket $!\n" unless $socket;
print "Server waiting for client connection on port 7777\n";
 
while(1) {
    # waiting for a new client connection
    my $client_socket = $socket->accept();
 
    # get information about a newly connected client
    my $client_address = $client_socket->peerhost();
    my $client_port = $client_socket->peerport();
    print "connection from $client_address:$client_port\n";

    # reading data from the client socket 
    my $data = "";

    # number in characters - if I'm not mistaking
    # TEXT in mysql can hold around 22000 characters
    # so that should do
    $client_socket->recv($data, 30000);

    @resultSplit = split(/::/, $data);

    print "Operation: $resultSplit[0]\n";
    
    my $insert = 0;
    if ($resultSplit[0] eq "insert") {
        $insert = 1;
    }

    print "SQL query: $resultSplit[1]\n";

    $sth = $dbh->prepare($resultSplit[1]); 
    $sth->execute or die "SQL Error: $DBI::errstr\n";

    if (!$insert) {
        $rownumber = $sth->rows;

        $client_socket->send("$rownumber\n");

        # write response data to the connected client
        while (@row = $sth->fetchrow_array) {
            $data = "@row\n";
            $client_socket->send($data);
        }
    }
 
    # notify client that response has been sent
    shutdown($client_socket, 1);
    print "Request handled successfully ...\n";
    print "Server waiting for client ...\n";

}
 
$socket->close();
