DROP DATABASE IF EXISTS webserver;

CREATE DATABASE webserver;

use webserver;

CREATE TABLE `components` (
  `id_comp` int(11) NOT NULL AUTO_INCREMENT,
  `name` varchar(45) DEFAULT NULL,
  PRIMARY KEY (`id_comp`)
) ENGINE=InnoDB AUTO_INCREMENT=0 DEFAULT CHARSET=latin1;

CREATE TABLE `architectures` (
  `id_arch` int(11) NOT NULL AUTO_INCREMENT,
  `description` TEXT DEFAULT NULL,
  PRIMARY KEY (`id_arch`)
) ENGINE=InnoDB AUTO_INCREMENT=0 DEFAULT CHARSET=latin1;

CREATE TABLE `arch_comp` (
  `id_comp` int(11) DEFAULT NULL,
  `id_arch` int(11) DEFAULT NULL,
  KEY `fk_comp_idx` (`id_comp`),
  KEY `fk_arch_idx` (`id_arch`),
  CONSTRAINT `fk_arch` FOREIGN KEY (`id_arch`) REFERENCES `architectures` (`id_arch`) ON DELETE NO ACTION ON UPDATE NO ACTION,
  CONSTRAINT `fk_comp` FOREIGN KEY (`id_comp`) REFERENCES `components` (`id_comp`) ON DELETE NO ACTION ON UPDATE NO ACTION
) ENGINE=InnoDB DEFAULT CHARSET=latin1;

#we store the average collected between "started" and "finished"
CREATE TABLE `events` (
  `id_event` int(11) NOT NULL AUTO_INCREMENT,
  `type` varchar(200) DEFAULT NULL,
  `started` timestamp NULL DEFAULT NULL,
  `finished` timestamp NULL DEFAULT NULL,
  `id_comp` int(11) DEFAULT NULL,
  `id_arch` int(11) DEFAULT NULL,
  `quantifier` double DEFAULT NULL,
  `counter` bigint(20) DEFAULT '0',
  PRIMARY KEY (`id_event`),
  KEY `fk_event_comp_idx` (`id_comp`),
  KEY `fk_event_arch_idx` (`id_arch`),
  CONSTRAINT `fk_event_arch` FOREIGN KEY (`id_arch`) REFERENCES `architectures` (`id_arch`) ON DELETE NO ACTION ON UPDATE NO ACTION,
  CONSTRAINT `fk_event_comp` FOREIGN KEY (`id_comp`) REFERENCES `components` (`id_comp`) ON DELETE NO ACTION ON UPDATE NO ACTION
) ENGINE=InnoDB AUTO_INCREMENT=0 DEFAULT CHARSET=latin1;

#we store the average collected between "started" and "finished"
CREATE TABLE `metrics` (
  `id_metric` int(11) NOT NULL AUTO_INCREMENT,
  `name` varchar(45) DEFAULT NULL,
  `value` double DEFAULT NULL,
  `high` tinyint(4) DEFAULT NULL,
  `started` timestamp NULL DEFAULT NULL,
  `finished` timestamp NULL DEFAULT NULL,
  `id_comp` int(11) DEFAULT NULL,
  `id_arch` int(11) DEFAULT NULL,
  `lead` tinyint(4) DEFAULT '0',
  `counter` int(11) DEFAULT NULL,
  PRIMARY KEY (`id_metric`),
  KEY `fk_metric_arch_idx` (`id_arch`),
  KEY `fk_metric_comp_idx` (`id_comp`),
  CONSTRAINT `fk_metric_arch` FOREIGN KEY (`id_arch`) REFERENCES `architectures` (`id_arch`) ON DELETE NO ACTION ON UPDATE NO ACTION,
  CONSTRAINT `fk_metric_comp` FOREIGN KEY (`id_comp`) REFERENCES `components` (`id_comp`) ON DELETE NO ACTION ON UPDATE NO ACTION
) ENGINE=InnoDB AUTO_INCREMENT=0 DEFAULT CHARSET=latin1;

CREATE TABLE `monitoring` (
  `id_monitor` int(11) NOT NULL AUTO_INCREMENT,
  `started` timestamp NULL DEFAULT NULL,
  `finished` timestamp NULL DEFAULT NULL,
  `id_arch` int(11) DEFAULT NULL,
  `event_type` varchar(200) DEFAULT NULL,
  `average_metric` double DEFAULT NULL,
  `num_req` bigint(20) DEFAULT NULL,
  `event_quantifier` double DEFAULT NULL,
  `event_counter` double DEFAULT NULL,
  PRIMARY KEY (`id_monitor`),
  KEY `fk_monitoring_arch_idx` (`id_arch`),
  CONSTRAINT `fk_monitoring_arch` FOREIGN KEY (`id_arch`) REFERENCES `architectures` (`id_arch`) ON DELETE NO ACTION ON UPDATE NO ACTION
) ENGINE=InnoDB AUTO_INCREMENT=0 DEFAULT CHARSET=latin1;

CREATE TABLE `pattern` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `type` varchar(200) DEFAULT NULL,
  `variation` int(11) DEFAULT NULL,
  `max_quantifier` double DEFAULT NULL,
  `min_quantifier` double DEFAULT NULL,
  `avg_quantifier` double DEFAULT NULL,
  `max_counter` bigint(20) DEFAULT NULL,
  `min_counter` bigint(20) DEFAULT NULL,
  `avg_counter` bigint(20) DEFAULT NULL,
  `num_archs` int(11) DEFAULT NULL,
  `min_metric` double DEFAULT NULL,
  `max_metric` double DEFAULT NULL,
  `avg_metric` double DEFAULT NULL,
  `started` timestamp NULL DEFAULT NULL,
  `finished` timestamp NULL DEFAULT NULL,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=0 DEFAULT CHARSET=latin1;

CREATE TABLE `pattern_aggregator` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `pattern_id` int(11) DEFAULT NULL,
  `pattern_table_id` int(11) DEFAULT NULL,
  PRIMARY KEY (`id`),
  KEY `fk_pattern_aggregator_1_idx` (`pattern_table_id`),
  CONSTRAINT `fk_pattern_aggregator_1` FOREIGN KEY (`pattern_table_id`) REFERENCES `pattern` (`id`) ON DELETE NO ACTION ON UPDATE NO ACTION
) ENGINE=InnoDB AUTO_INCREMENT=0 DEFAULT CHARSET=latin1;

CREATE TABLE `learning` (
  `id_learning` int(11) NOT NULL AUTO_INCREMENT,
  `id_arch` int(11) DEFAULT NULL,
  `id_pattern` int(11) DEFAULT NULL,
  `metric` double DEFAULT NULL,
  PRIMARY KEY (`id_learning`),
  KEY `fk_learning_arch_idx` (`id_arch`),
  CONSTRAINT `fk_learning_arch` FOREIGN KEY (`id_arch`) REFERENCES `architectures` (`id_arch`) ON DELETE NO ACTION ON UPDATE NO ACTION
) ENGINE=InnoDB AUTO_INCREMENT=0 DEFAULT CHARSET=latin1;
